@@ -1,10 +1,11 @@
 from requests import Session
 from requests.exceptions import RequestException
 from requests.auth import HTTPBasicAuth
+from urllib.parse import urljoin
 
 
-class Medialoopster():
-    def __init__(self, url, user=None, password=None, verify=False):
+class Medialoopster:
+    def __init__(self, url: str, user: str = None, password: str = None, verify: bool = False):
         self.session = Session()
         self.session.verify = verify
 
@@ -19,11 +20,11 @@ class Medialoopster():
 
     def __exit__(self, *args):
         """Clean up."""
-        self.close()
+        self.session.close()
 
     def ping(self):
         try:
-            response = self.session.get(self.url + "ping/")
+            response = self.session.get(f"{self.url}ping/")
             response.raise_for_status()
         except RequestException:
             return False
@@ -31,7 +32,10 @@ class Medialoopster():
         return True
 
     def asset_import(self, production=None, asset_type=None, move_asset=False, name=None, description=None, approval=0,
-                     path_file=None, meta_field_store={}):
+                     path_file=None, meta_field_store=None):
+        if meta_field_store is None:
+            meta_field_store = {}
+
         request = {
             "production": production,
             "type": asset_type,
@@ -47,27 +51,19 @@ class Medialoopster():
             }
         }
 
-        response = self.session.post(self.url + "asset/import/", json=request)
+        response = self.session.post(f"{self.url}asset/import/", json=request)
         response.raise_for_status()
 
         response_json = response.json()
 
         return response_json.get("asset_import_id", None)
 
-    def get_url(self, asset_type="videoassets"):
-        try:
-            response = self.session.get(url=self.url)
-            response.raise_for_status()
-
-            response_json = response.json()
-        except RequestException:
-            return None
-
-        return response_json.get(asset_type, None)
-
-    def get_from_api(self, asset_type="videoassets", url=None):
+    def get_from_api(self, asset_type="videoassets", url=None, with_sequences=True):
         if url is None:
-            url = self.get_url(asset_type=asset_type)
+            url = f"{self.url}{asset_type}"
+
+            if asset_type == "videoassets" and with_sequences is True:
+                url = f"{url}?with_sequences=true"
 
         while url:
             try:
@@ -81,17 +77,16 @@ class Medialoopster():
             else:
                 url = None
 
-            for response_json in response.json():
-                yield response_json
+            yield from response.json()
 
     def get_productions(self):
         return self.get_from_api(asset_type="productions")
 
-    def get_videoassets(self):
-        return self.get_from_api(asset_type="videoassets")
+    def get_videoassets(self, with_sequences=True):
+        return self.get_from_api(asset_type="videoassets", with_sequences=with_sequences)
 
     def get_asset(self, asset_id, asset_type="videoassets", with_sequences=True):
-        url = f"{self.get_url(asset_type=asset_type)}{asset_id}/"
+        url = f"{self.url}{asset_type}/{asset_id}/"
 
         if asset_type == "videoassets" and with_sequences is True:
             url = f"{url}?with_sequences=true"
@@ -99,9 +94,7 @@ class Medialoopster():
         response = self.session.get(url=url)
         response.raise_for_status()
 
-        response_json = response.json()
-
-        return response_json
+        return response.json()
 
     def search_meta_field_store(self, field: str = None, value: str = None, asset_type: str = "videoassets"):
         assets = []
@@ -115,3 +108,54 @@ class Medialoopster():
                     assets.append(asset.get("id"))
 
         return assets
+
+    def archiv_asset(self, content_type_id, asset_id, archive_type, archive_connector):
+        url = urljoin(self.url, "archiveactivities/")
+
+        request = [{
+            "content_type_id": content_type_id,
+            "asset_id": asset_id,
+            "archive_type": archive_type,
+            "archive_connector": archive_connector,
+        }]
+
+        response = self.session.post(url, json=request)
+        response.raise_for_status()
+
+        return response.json()
+
+    def disapprove_asset(self, asset_id, asset_type="videoassets"):
+        url = f"{self.url}{asset_type}/{asset_id}/"
+
+        request = {
+            "status_approval": 0
+        }
+
+        response = self.session.patch(url, json=request)
+        response.raise_for_status()
+
+        return response.json()
+
+    def set_date_del_asset(self, asset_id, date_del, asset_type="videoassets"):
+        url = f"{self.url}{asset_type}/{asset_id}/"
+
+        request = {
+            "date_del": date_del
+        }
+
+        response = self.session.patch(url, json=request)
+        response.raise_for_status()
+
+        return response.json()
+
+    def edit_meta_field_store(self, asset_id, meta_field_store, asset_type="videoassets"):
+        url = f"{self.url}{asset_type}/{asset_id}/"
+
+        request = {
+            "meta_field_store": meta_field_store
+        }
+
+        response = self.session.patch(url, json=request)
+        response.raise_for_status()
+
+        return response.json()
